<?php
namespace Teknisa\FAI\Modules\Relatorio\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class FAI01400FaturamentoDeRefeicoesPorFilial extends \Zeedhi\Framework\Controller\Report {

	protected $qr2Strategy;
	protected $environment;
	protected $newCode;
	protected $connection;
	protected $crudService;

	public function __construct($reportService, $qr2Strategy, $environment, $newCode, $connection, $crudService) {
		$this->qr2Strategy  = $qr2Strategy;
		$this->environment  = $environment;
		$this->newCode      = $newCode;
		$this->connection  	= $connection;
		$this->crudService  = $crudService;

		parent::__construct($reportService);
	}

	protected $reportMapping = array (
		'FAI01400' =>
		array (
			'strategy' => 'QR2',
			'parameterMapping' =>
			array (
				'DATAINI'    => 'P_DTINICIAL',
				'DATAFIN'    => 'P_DTFINAL',
				'NRAPURACGD' => 'P_NRAPURACGD',
				'CDCLIENTE'  => 'P_CDCLIENTE'
			)
		),
		'FAI01401' =>
		array (
			'strategy' => 'QR2',
			'parameterMapping' =>
			array (
				'DATAINICIAL'   => 'P_DTINICIAL',
				'DATAFINAL'   => 'P_DTFINAL',
				'CDFILIAL'  => 'P_CDFILIAL',
				'TCDFILIAL'  => 'P_TCDFILIAL',
				'CDCLIENTE' => 'P_CDCLIENTE',
				'TCDCLIENTE' => 'P_TCDCLIENTE',
				'DATAINI'   => 'P_DTINI',
				'DATAFIN'   => 'P_DTFIN'
			)
		),
		'FAI01402' =>
		array (
			'strategy' => 'QR2',
			'parameterMapping' =>
			array (
				'DATAINICIAL'   => 'P_DTINICIAL',
				'DATAFINAL'   => 'P_DTFINAL',
				'CDFILIAL'  => 'P_CDFILIAL',
				'TCDFILIAL'  => 'P_TCDFILIAL',
				'CDCLIENTE' => 'P_CDCLIENTE',
				'TCDCLIENTE' => 'P_TCDCLIENTE',
				'DATAINI'   => 'P_DTINI',
				'DATAFIN'   => 'P_DTFIN'
			)
		),
		'FAI01403' =>
		array (
			'strategy' => 'QR2',
			'parameterMapping' =>
			array (
				'DATAINICIAL'   => 'P_DTINICIAL',
				'DATAFINAL'   => 'P_DTFINAL',
				'CDFILIAL'  => 'P_CDFILIAL',
				'TCDFILIAL'  => 'P_TCDFILIAL',
				'CDCLIENTE' => 'P_CDCLIENTE',
				'TCDCLIENTE' => 'P_TCDCLIENTE',
				'DATAINI'   => 'P_DTINI',
				'DATAFIN'   => 'P_DTFIN'
			)
		)
	);

	public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {

		$row = $request->getRow();
		$this->qr2Strategy->setQr2Type($row['QR2TYPE']);

		if($row['__report_name'] != 'FAI01400') {
			$row['TCDFILIAL'] = $row['ALL_CDFILIAL'] ? "T" : "";
			$row['TCDCLIENTE'] = $row['CLIENTE'] === 'T' ? "T" : "";
			$row['DATAINICIAL'] = $row['DATAINI'];
			$row['DATAFINAL'] = $row['DATAFIN'];
		}

		$row['NRAPURACGD'] = 'T'.$this->newCode->getCodeByTableName('NRAPURACGD', $this->environment->getNrOrg(), 1, 9);


		if($row['ALL_CDFILIAL']){
			$this->mountDataFilial($row);
		} else {
			foreach ($row['CDFILIAL'] as $filial) {
				$this->executeProcedure($row, $filial);
			}
			$row['CDFILIAL'] = '_'.implode('_', $row['CDFILIAL']).'_';
		}
	}

	private function executeProcedure($row, $cdfilial){
		$procedure = new StoredProcedure($this->connection, 'APURAC_RELGESDIRETA');
		$procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $cdfilial));
		$procedure->addParam(new Param('P_DATAINI', Param::PARAM_INPUT, $row['DATAINI']));
		$procedure->addParam(new Param('P_DATAFIN', Param::PARAM_INPUT, $row['DATAFIN']));
		$procedure->addParam(new Param('P_NRAPURACGD', Param::PARAM_INPUT, $row['NRAPURACGD']));
		$procedure->addParam(new Param('P_CDCLIENTE', Param::PARAM_INPUT, $row['CDCLIENTE']));
		$procedure->execute();
	}

	private function mountDataFilial($row){
		$conditions = array(
			array(
				"columnName" => "CDOPERADOR",
				"operator" => "=",
				"value" => $this->environment->getCdOperador()
			)
		);
		$dataSet = $this->crudService->getFilteredDataSet('filioper', $conditions)->getRows();
		foreach($dataSet as $filial){
			$this->executeProcedure($row, $filial['CDFILIAL']);
		}
	}
}
